#<profile4xi profile = "03" /> 
# -------------------------------------------------------------------------------------------------
#<program4xi program = "simplequeue4xi.py">
#<abstract4xi abstract = "provides class SimpleQueue4Xi"/>
#<refer4xi reference = "queues.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'SimpleQueue4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import time
    import weakref
      
except:
    pass
try:                 # rare ones
    import collections    
    from queue import Empty, Full
    import atexit              
except:
    pass 
try:                 # imported-as ones 
    pass          
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.queues import SimpleQueue
except:
    print("simplequeue4xi.py: *** failed in from multiprocessing.queues import SimpleQueue ***")
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi

from assert_spawning4xi import assert_spawning4xi
from constant_system4xi import get_windows_system4xi
from lock4xi import Lock4Xi
from pipe4xi import Pipe4Xi
from print4xi import print4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#"
#/>





# =================================================================================================
#<class4xi class = "SimpleQueue4Xi" base = "object" child = "None"
#  synopsis = "a locked pipe"
#  usage = "
#     pool.py
#         def _setup_queues(self):
#             self._inqueue = SimpleQueue()
#             self._outqueue = SimpleQueue()
#     pool4xi.py:
#         self._inqueue = SimpleQueue4Xi()
#         self._outqueue = SimpleQueue4Xi()
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class SimpleQueue4Xi(object):
    '''
    SimpleQueue4Xi class: really just a locked pipe
    '''
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('empty',
                 'sense', 'reason', 'ace')          
    #"
    #/>
       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-SimpleQueue4Xi" usage = "initializes SimpleQueue4Xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------  
    def __init__(self):
        '''
        initializes SimpleQueue4Xi 
        '''
                                        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/7) - START
        # -----------------------------------------------------------------------------------------                 
             
        class2print = "SimpleQueue4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/7a) started" + "; init-of-SimpleQueue4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/7b) borning" + "; simplequeue2xi-of-SimpleQueue4Xi-borning: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        message2print = "{(Step 1/7c) inheriting" + "; object-inheriting-by-simplequeue2xi-of-SimpleQueue4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                                
                               
        # -----------------------------------------------------------------------------------------    
        # (Step 2/7) - PIPE
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 2/7a) passing" + "; False-passing-to-Pipe4Xi-by-init-of-SimpleQueue4Xi: " + str(False) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        try:                   
            self._reader, self._writer = Pipe4Xi(duplex=False)
            
            message2print = "{(Step 2/7b) set" + "; reader-of-simplequeue2xi-set-by-init-of-SimpleQueue4Xi: " + str(self._reader)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            message2print = "{(Step 2/7c) set" + "; writer-of-simplequeue2xi-set-by-init-of-SimpleQueue4Xi: " + str(self._writer)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        except:
            message2print = "{(Step 2/7d) yielded" + "; Pipe4Xi-yielded-by-init-of-SimpleQueue4Xi: " + str(Pipe4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------    
        # (Step 3/7) - RLOCK
        # ----------------------------------------------------------------------------------------- 
        try:
            self._rlock = Lock4Xi()
            
            message2print = "{(Step 2/7d) set" + "; rlock-of-simplequeue2xi-set-by-init-of-SimpleQueue4Xi: " + str(self._rlock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        except:
            message2print = "{(Step 2/7e) yielded" + "; Lock4Xi-yielded-by-init-of-SimpleQueue4Xi: " + str(Lock4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
             
        
        # -----------------------------------------------------------------------------------------    
        # (Step 4/7) - POLL
        # -----------------------------------------------------------------------------------------             
        self._poll = None
        try:
            self._poll = self._reader.poll    
            
            message2print = "{(Step 4/7a) set" + "; poll-of-simplequeue2xi-set-by-init-of-SimpleQueue4Xi: " + str(self._poll)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
        except:
            message2print = "{(Step 2/7e) yielded" + "; poll-yielded-by-init-of-SimpleQueue4Xi: " + str(self._poll)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                    
        # -----------------------------------------------------------------------------------------    
        # (Step 5/7) - WLOCK
        # -----------------------------------------------------------------------------------------                 
        self._wlock = None                         #dragonized  
        
        if sys.platform == get_windows_system4xi(): 
            self._wlock = None
            
            message2print = "{(Step 3/7a) set" + "; wlock-of-simplequeue2xi-set-by-init-of-SimpleQueue4Xi to: " + str(self._wlock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                            
        else:
            try:
                self._wlock = Lock4Xi()
                
                
                message2print = "{(Step 3/7b) set" + "; wlock-of-simplequeue2xi-set-by-init-of-SimpleQueue4Xi to: " + str(self._wlock)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            except:
                message2print = "{(Step 3/7c) yielded" + "; Lock4Xi-yielded-by-init-of-SimpleQueue4Xi: " + str(Lock4Xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
              
        
             
            
        # -----------------------------------------------------------------------------------------    
        # (Step 6/7) - METHODS
        # -----------------------------------------------------------------------------------------                 
        try:       
            self._make_methods()
            
            message2print = "{(Step 4/7a) performed" + "; make-methods-of-simplequeue2xi-performed-by-init-of-SimpleQueue4Xi: " + str(self._make_methods) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        except:
            message2print = "{(Step 4/7b) yielded" + "; make-methods-of-simplequeue2xi-yielded-by-init-of-SimpleQueue4Xi: " + str(self._make_methods)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        
        # -----------------------------------------------------------------------------------------    
        # (Step 7/7) - RETURN
        # ----------------------------------------------------------------------------------------- 
        
        message2print = "{(Step 5/7a) inherited" + "; object-inherited-by-simplequeue2xi-of-SimpleQueue4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                
        message2print = "{(Step 5/7b) activated" + "; simplequeue2xi-of-SimpleQueue4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                            
        message2print = "{(Step 5/7c) returning" + "; init-of-SimpleQueue4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
        return
                        
                                
    #</methodize4xi>
                     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "empty-of-SimpleQueue4Xi" usage = "returns whethet empty">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------  
    def empty(self):
        '''
        returns whether empty
        '''
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------           
        class2print = "SimpleQueue4Xi"
        method2print = "empty"
        
        message2print = "{started" + "; empty-of-SimpleQueue4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
               
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; poll-function-of-simplequeue2xi-returning-by-empty-of-SimpleQueue4Xi: " + str(self._poll) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        
        return not self._poll()
    #</methodize4xi>
    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getstate-of-SimpleQueue4Xi" usage = "gets state">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------  
    def __getstate__(self):
        '''
        gets state
        '''
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------           
        class2print = "SimpleQueue4Xi"
        method2print = "__getstate__"
        
        message2print = "{started" + "; getstate-of-SimpleQueue4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
     
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        try:
            assert_spawning4xi(self)
            
            message2print = "{performed" + "; assert-spawning4xi-performed-by-getstate-of-SimpleQueue4Xi: " + str(assert_spawning4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        except:
            message2print = "{yielded" + "; assert_spawning4xi-yielded-by-getstate-of-SimpleQueue4Xi: " + str(assert_spawning4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
              
                        
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; reader-of-simplequeue2xi-returning-by-getstate-of-SimpleQueue4Xi: " + str(self._reader) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                     
        message2print = "{returning" + "; writer-of-simplequeue2xi-returning-by-getstate-of-SimpleQueue4Xi: " + str(self._writer) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        
        message2print = "{returning" + "; rlock-of-simplequeue2xi-returning-by-getstate-of-SimpleQueue4Xi: " + str(self._rlock) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        
        message2print = "{returning" + "; wlock-of-simplequeue2xi-returning-by-getstate-of-SimpleQueue4Xi: " + str(self._wlock) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
        return (self._reader, self._writer, self._rlock, self._wlock)
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "setstate-of-SimpleQueue4X" usage = "sets state">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------  
    def __setstate__(self, state):
        '''
        sets state
        '''
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------           
        class2print = "SimpleQueue4Xi"
        method2print = "__setstate__"
        
        message2print = "{started" + "; setstate-of-SimpleQueue4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{passed" + "; state-passed-to-setstate-of-SimpleQueue4Xi: " + str(state) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
   
        try:
            (self._reader, self._writer, self._rlock, self._wlock) = state
                        
            message2print = "{set" + "; reader-of-simplequeue2xi-set-by-setstate-of-SimpleQueue4Xi to: " + str(self._reader)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            message2print = "{set" + "; writer-of-simplequeue2xi-set-by-setstate-of-SimpleQueue4Xi to: " + str(self._writer)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            message2print = "{set" + "; rlock-of-simplequeue2xi-set-by-setstate-of-SimpleQueue4Xi to: " + str(self._rlock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                                
            
            message2print = "{set" + "; wlock-of-simplequeue2xi-set-by-setstate-of-SimpleQueue4Xi to: " + str(self._wlock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{yielded" + "; state-yielded-by-setstate-of-SimpleQueue4Xi: " + str(state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                               
        
        try:    
            self._make_methods()
            
            message2print = "{performed" + "; make-methods-of-simplequeue2xi-performed-by-setstate-of-SimpleQueue4Xi: " + str(self._make_methods) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        except:
            message2print = "{yielded" + "; make-methods-of-simplequeue2xi-yielded-by-setstate-of-SimpleQueue4Xi: " + str(self._make_methods)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

            
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; setstate-of-SimpleQueue4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
        return
                        
                                
          
    #</methodize4xi>
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "make-methods-of-SimpleQueue4Xi" usage = "makes method">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------  
    def _make_methods(self):
        '''
        makes methods
        '''
        # -----------------------------------------------------------------------------------------    
        # (Step 1/8)
        # -----------------------------------------------------------------------------------------           
        class2print = "SimpleQueue4Xi"
        method2print = "_make_methods"
        
        message2print = "{(Step 1/8) started" + "; make-methods-of-SimpleQueue4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        # -----------------------------------------------------------------------------------------    
        # (Step 2/8)
        # ----------------------------------------------------------------------------------------- 
                        
        message2print = "{(Step 2/8a) using" + "; reader-recv-of-simplequeue2xi-using-by-make-methods-of-SimpleQueue4Xi: " + str(self._reader.recv)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
        try:
            recv = self._reader.recv
            
            message2print = "{(Step 2/8b) set" + "; recv-set-by-make-methods-of-SimpleQueue4Xi to: " + str(recv)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 2/8c) yielded" + "; reader-recv-of-simplequeue2xi-yielded-by-make-methods-of-SimpleQueue4Xi: " + str(self._reader.recv)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
            
   
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
                                  
        message2print = "{(Step 2/8c) using" + "; rlock-acquire-of-simplequeue2xi-using-by-make-methods-of-SimpleQueue4Xi: " + str(self._rlock.acquire)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                          
                        
        message2print = "{(Step 2/8d) using" + "; rlock-release-of-simplequeue2xi-using-by-make-methods-of-SimpleQueue4Xi: " + str(self._rlock.release)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
          
        try:
            racquire, rrelease = self._rlock.acquire, self._rlock.release
            
            message2print = "{(Step 2/8e) set" + "; racquire-set-by-make-methods-of-SimpleQueue4Xi to: " + str(racquire)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            
            message2print = "{(Step 2/8f) set" + "; rrelease-set-by-make-methods-of-SimpleQueue4Xi to: " + str(rrelease)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 2/8g) yielded" + "; rlock-acquire-of-simplequeue2xi-yielded-by-make-methods-of-SimpleQueue4Xi: " + str(self._rlock.acquire)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                      
            message2print = "{(Step 2/8h) yielded" + "; rlock-release-of-simplequeue2xi-yielded-by-make-methods-of-SimpleQueue4Xi: " + str(self._rlock.release)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------    
        #<function4xi function = "get" usage = "
        # embedded get
        #">     
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # -----------------------------------------------------------------------------------------                 
        
        def get():
         
            # -------------------------------------------------------------------------------------    
            # (Step 3/8)    
            # -------------------------------------------------------------------------------------                 
                
            message2print = "{(Step 3/8a) started" + "; get-of-SimpleQueue4Xi-started: " + str(get)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
                                
            racquire()
            
            message2print = "{(Step 3/8b) performed" + "; racquire-performed-by-get-of-SimpleQueue4Xi: " + str(racquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
            
            try:
                message2print = "{(Step 3/8c) returning" + "; recv-function-returning-by-get-of-SimpleQueue4Xi: " + str(recv) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
                                
            
                return recv()
            
                
            finally:
                rrelease()
                
                message2print = "{(Step 3/8d) performed" + "; rrelease-performed-by-get-of-SimpleQueue4Xi: " + str(rrelease) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
        #</function4xi>
         
        # -----------------------------------------------------------------------------------------    
        # (Step 4/8)    
        # -----------------------------------------------------------------------------------------                 
                  
        self.get = get

        message2print = "{(Step 4/8) set" + "; get-of-simplequeue2xi-set-by-make-methods-of-SimpleQueue4Xi to: " + str(self.get)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
                                 
        
        # -----------------------------------------------------------------------------------------    
        # (Step 5/8)
        # -----------------------------------------------------------------------------------------                 
   
        if self._wlock is None:
            
            self.put = self._writer.send
            
            message2print = "{(Step 5/8) set" + "; put-of-simplequeue2xi-set-by-make-methods-of-SimpleQueue4Xi to: " + str(self.put)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                            
            
        else:
            # -------------------------------------------------------------------------------------    
            # (Step 6/8)
            # -------------------------------------------------------------------------------------           
            send = self._writer.send
            
            message2print = "{(Step 6/8a) set" + "; send-set-by-make-methods-of-SimpleQueue4Xi to: " + str(send)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                            
            
            wacquire, wrelease = self._wlock.acquire, self._wlock.release
            
            message2print = "{(Step 6/8b) set" + "; wacquire-set-by-make-methods-of-SimpleQueue4Xi to: " + str(wacquire)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            message2print = "{(Step 6/8c) set" + "; wrelease-set-by-make-methods-of-SimpleQueue4Xi to: " + str(wrelease)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            # -------------------------------------------------------------------------------------    
            #<function4xi function = "put" usage = "
            # embedded put
            #"> 
            #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
            # -------------------------------------------------------------------------------------                 

            # -------------------------------------------------------------------------------------    
            # (Step 7/8)    
            # -------------------------------------------------------------------------------------              
                           
            def put(obj):

                # ---------------------------------------------------------------------------------    
                #     
                # ---------------------------------------------------------------------------------              
                                      
                message2print = "{(Step 7/8b) started" + "; put-of-SimpleQueue4Xi-started: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                                    
                
                message2print = "{(Step 7/8c) passed" + "; object-passed-to-put-of-SimpleQueue4Xi: " + str(obj) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  

                # ---------------------------------------------------------------------------------    
                #     
                # ---------------------------------------------------------------------------------              
                  
                wacquire()
                
                message2print = "{(Step 7/8d) performed" + "; wacquire-performed-by-put-of-SimpleQueue4Xi: " + str(wacquire) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             

                # ---------------------------------------------------------------------------------    
                #     
                # ---------------------------------------------------------------------------------              
                                      
                try:
                    message2print = "{(Step 7/8e) passing" + "; object-passing-to-send-by-put-of-SimpleQueue4Xi: " + str(obj) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                    
                    message2print = "{(Step 7/8f) returning" + "; send-function-returning-by-put-of-SimpleQueue4Xi: " + str(send) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)  
                                                    
                
                    return send(obj)

                # ---------------------------------------------------------------------------------    
                #     
                # ---------------------------------------------------------------------------------              
                      
                finally:
                    wrelease()
                    
                    message2print = "{(Step 7/8g) performed" + "; wrelease-performed-by-put-of-SimpleQueue4Xi: " + str(wrelease) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 

                # ---------------------------------------------------------------------------------    
                #     
                # ---------------------------------------------------------------------------------              
                                      
            #</function4xi>
                                                
            self.put = put

            # -------------------------------------------------------------------------------------    
            # (Step 7/8)    
            # -------------------------------------------------------------------------------------              
                 
            message2print = "{(Step 7/8h) set" + "; put-of-simplequeue2xi-set-by-make-methods-of-SimpleQueue4Xi to: " + str(self.put)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
 
            
        # -----------------------------------------------------------------------------------------    
        # (Step 8/8)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 8/8) returning" + "; make-methods-of-SimpleQueue4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
        return
                                
                                   
    #</methodize4xi>
    
                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-SimpleQueue4Xi" usage = "senses SimpleQueue4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses SimpleQueue4Xi class objects
        '''
 
        
        class2print = 'SimpleQueue4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-SimpleQueue4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; SimpleQueue4Xi-sensing-by-sense-of-SimpleQueue4Xi: " + str(SimpleQueue4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2x-got-by-sense-of-SimpleQueue4Xi: " + str(kb2x)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['reader-of-simplequeue2xi'] = self._reader
            kb2xi['writer-of-simplequeue2xi'] = self._writer            
            kb2xi['rlock-of-simplequeue2xi'] = self._rlock
            kb2xi['wlock-of-simplequeue2xi'] = self._wlock      
                                 
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-SimpleQueue4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                    
        
        
      
        message2print = "{sensed" + "; SimpleQueue4Xi-sensed-by-sense-of-SimpleQueue4Xi: " + str(SimpleQueue4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-SimpleQueue4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-SimpleQueue4Xi" usage = "reasons SimpleQueue4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons SimpleQueue4Xi class objects
        '''

        
        class2print = 'SimpleQueue4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-SimpleQueue4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; SimpleQueue4Xi-reasoning-by-reason-of-SimpleQueue4Xi: " + str(SimpleQueue4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-SimpleQueue4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; SimpleQueue4Xi-reasoned-by-reason-of-SimpleQueue4Xi: " + str(SimpleQueue4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-SimpleQueue4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                              
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-SimpleQueue4Xi" usage = "aces SimpleQueue4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces SimpleQueue4Xi class objects 
        '''
 
        
        
        class2print = 'SimpleQueue4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-SimpleQueue4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; SimpleQueue4Xi-acing-by-ace-of-SimpleQueue4Xi: " + str(SimpleQueue4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                    
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-SimpleQueue4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; SimpleQueue4Xi-aced-by-ace-of-SimpleQueue4Xi: " + str(SimpleQueue4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-SimpleQueue4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of simplequeue4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>
